@@ -202,9 +202,9 @@ nautilus_drag_build_selection_list (GtkSelectionData *data)
 	return result;
 }
 
-
-gboolean
-nautilus_drag_items_local (const char *target_uri_string, const GList *selection_list)
+static gboolean
+nautilus_drag_file_local_internal (const char *target_uri_string,
+				   const char *first_source_uri)
 {
 	/* check if the first item on the list has target_uri_string as a parent
 	 * FIXME:
@@ -214,15 +214,12 @@ nautilus_drag_items_local (const char *target_uri_string, const GList *selection
 	GFile *target, *item, *parent;
 	gboolean result;
 
-	/* must have at least one item */
-	g_assert (selection_list);
-
 	result = FALSE;
 
 	target = g_file_new_for_uri (target_uri_string);
 
 	/* get the parent URI of the first item in the selection */
-	item = g_file_new_for_uri (((NautilusDragSelectionItem *)selection_list->data)->uri);
+	item = g_file_new_for_uri (first_source_uri);
 	parent = g_file_get_parent (item);
 	g_object_unref (item);
 	
@@ -232,6 +229,27 @@ nautilus_drag_items_local (const char *target_uri_string, const GList *selection
 	}
 	
 	return result;
+}	
+
+gboolean
+nautilus_drag_uris_local (const char *target_uri,
+			  const GList *source_uri_list)
+{
+	/* must have at least one item */
+	g_assert (source_uri_list);
+	
+	return nautilus_drag_file_local_internal (target_uri, source_uri_list->data);
+}
+
+gboolean
+nautilus_drag_items_local (const char *target_uri_string,
+			   const GList *selection_list)
+{
+	/* must have at least one item */
+	g_assert (selection_list);
+
+	return nautilus_drag_file_local_internal (target_uri_string,
+						  ((NautilusDragSelectionItem *)selection_list->data)->uri);
 }
 
 gboolean
@@ -347,6 +365,7 @@ nautilus_drag_default_drop_action_for_icons (GdkDragContext *context,
 {
 	gboolean same_fs;
 	gboolean target_is_source_parent;
+	const char *dropped_uri;
 	GFile *target, *dropped;
 	GdkDragAction actions;
 
@@ -368,6 +387,8 @@ nautilus_drag_default_drop_action_for_icons (GdkDragContext *context,
 		return;
 	}
 	
+	dropped_uri = ((NautilusDragSelectionItem *)items->data)->uri;
+	
 	/*
 	 * Check for trash URI.  We do a find_directory for any Trash directory.
 	 * Passing 0 permissions as gnome-vfs would override the permissions
@@ -381,25 +402,33 @@ nautilus_drag_default_drop_action_for_icons (GdkDragContext *context,
 
 		return;
 
-	} else if (g_str_has_prefix (target_uri_string, NAUTILUS_COMMAND_SPECIFIER)
-			|| g_str_has_prefix (target_uri_string, NAUTILUS_DESKTOP_COMMAND_SPECIFIER)) {
+	} else if (g_str_has_prefix (target_uri_string, NAUTILUS_COMMAND_SPECIFIER) ||
+		   g_str_has_prefix (target_uri_string, NAUTILUS_DESKTOP_COMMAND_SPECIFIER) ) {
 		if (actions & GDK_ACTION_MOVE) {
 			*action = GDK_ACTION_MOVE;
 		}
 		return;
 	} else if (eel_uri_is_desktop (target_uri_string)) {
 		target = nautilus_get_desktop_location ();
+		if (eel_uri_is_desktop (dropped_uri)) {
+			/* Only move to Desktop icons */
+			if (actions & GDK_ACTION_MOVE) {
+				*action = GDK_ACTION_MOVE;
+			}
+			
+			return;
+		}
 	} else {
 		target = g_file_new_for_uri (target_uri_string);
 	}
 
 	/* Compare the first dropped uri with the target uri for same fs match. */
-	dropped = g_file_new_for_uri (((NautilusDragSelectionItem *)items->data)->uri);
+	dropped = g_file_new_for_uri (dropped_uri);
 	same_fs = check_same_fs (target, dropped);
-	target_is_source_parent = FALSE;
-	target_is_source_parent = g_file_contains_file (target, dropped);	
+	target_is_source_parent = g_file_contains_file (target, dropped);
 	
-	if (same_fs || target_is_source_parent) {
+	if (same_fs || target_is_source_parent ||
+	    g_file_has_uri_scheme (dropped, "trash")) {
 		if (actions & GDK_ACTION_MOVE) {
 			*action = GDK_ACTION_MOVE;
 		} else {
