@@ -31,10 +31,10 @@
 #include <string.h>
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-glib-extensions.h>
+#include <eel/eel-gdk-pixbuf-extensions.h>
 #include <gtk/gtktreednd.h>
 #include <gtk/gtktreesortable.h>
 #include <glib/gi18n.h>
-#include <libnautilus-private/nautilus-icon-factory.h>
 #include <libnautilus-private/nautilus-dnd.h>
 
 #if GLIB_CHECK_VERSION (2, 13, 0)
@@ -48,14 +48,18 @@ enum {
 	LAST_SIGNAL
 };
 
+static GQuark attribute_name_q,
+	attribute_modification_date_q,
+	attribute_date_modified_q;
+
 /* msec delay after Loading... dummy row turns into (empty) */
 #define LOADING_TO_EMPTY_DELAY 100
 
 static guint list_model_signals[LAST_SIGNAL] = { 0 };
 
 static int fm_list_model_file_entry_compare_func (gconstpointer a,
-				       gconstpointer b,
-				       gpointer      user_data);
+						  gconstpointer b,
+						  gpointer      user_data);
 
 static GObjectClass *parent_class;
 
@@ -66,7 +70,7 @@ struct FMListModelDetails {
 
 	int stamp;
 
-	char *sort_attribute;
+	GQuark sort_attribute;
 	GtkSortType order;
 
 	gboolean sort_directories_first;
@@ -250,11 +254,12 @@ fm_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int column
 	int icon_size;
 	guint emblem_size;
 	NautilusZoomLevel zoom_level;
-	char *modifier;
-	GList *emblem_icons;
+	GList *emblem_pixbufs;
 	NautilusFile *parent_file;
-	EelStringList *emblems_to_ignore;
-
+	char *emblems_to_ignore[3];
+	int i;
+	NautilusFileIconFlags flags;
+	
 	model = (FMListModel *)tree_model;
 
 	g_return_if_fail (model->details->stamp == iter->stamp);
@@ -287,7 +292,7 @@ fm_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int column
 			zoom_level = fm_list_model_get_zoom_level_from_column_id (column);
 			icon_size = nautilus_get_icon_size_for_zoom_level (zoom_level);
 
-			modifier = NULL;
+			flags = NAUTILUS_FILE_ICON_FLAGS_USE_THUMBNAILS;
 			if (model->details->drag_view != NULL) {
 				GtkTreePath *path_a, *path_b;
 				
@@ -298,19 +303,15 @@ fm_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int column
 					path_b = gtk_tree_model_get_path (tree_model, iter);
 
 					if (gtk_tree_path_compare (path_a, path_b) == 0) {
-						modifier = "accept";
+						flags |= NAUTILUS_FILE_ICON_FLAGS_FOR_DRAG_ACCEPT;
 					}
 						
 					gtk_tree_path_free (path_a);
 					gtk_tree_path_free (path_b);
 				}
 			}
-			
-			if (nautilus_file_has_open_window (file)) {
-				modifier = "visiting";
-			}
-			
-			icon = nautilus_icon_factory_get_pixbuf_for_file (file, modifier, icon_size, TRUE);
+
+			icon = nautilus_file_get_icon_pixbuf (file, icon_size, TRUE, flags);
   
 			g_value_set_object (value, icon);
 			g_object_unref (icon);
@@ -327,38 +328,34 @@ fm_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int column
 
 		if (file != NULL) {
 			parent_file = nautilus_file_get_parent (file);
-			emblems_to_ignore = eel_string_list_new_from_string (NAUTILUS_FILE_EMBLEM_NAME_TRASH, TRUE);
+			i = 0;
+			emblems_to_ignore[i++] = NAUTILUS_FILE_EMBLEM_NAME_TRASH;
 			if (parent_file) {
 				if (!nautilus_file_can_write (parent_file)) {
-					eel_string_list_prepend (emblems_to_ignore, NAUTILUS_FILE_EMBLEM_NAME_CANT_WRITE);
+					emblems_to_ignore[i++] = NAUTILUS_FILE_EMBLEM_NAME_CANT_WRITE;
 				}
 				nautilus_file_unref (parent_file);
 			}
+			emblems_to_ignore[i++] = NULL;
+			
 			zoom_level = fm_list_model_get_zoom_level_from_emblem_column_id (column);
 			icon_size = nautilus_get_icon_size_for_zoom_level (zoom_level);
-			emblem_size = nautilus_icon_factory_get_emblem_size_for_icon_size (icon_size);
+			emblem_size = nautilus_icon_get_emblem_size_for_icon_size (icon_size);
 			/* Special case default icon size here. This works semi-ok, since we
 			   only show one emblem for the list view anyway */
 			if (emblem_size == 0 && icon_size >= 24) {
 				emblem_size = 16;
 			}
-			emblem_icons = NULL;
 			if (emblem_size != 0) {
-				emblem_icons = nautilus_icon_factory_get_emblem_icons_for_file (file, emblems_to_ignore);
-			}
-			eel_string_list_free (emblems_to_ignore);
-
-			if (emblem_icons != NULL) {
-				icon = nautilus_icon_factory_get_pixbuf_for_icon (
-					emblem_icons->data, NULL, emblem_size,
-					NULL, NULL, TRUE, FALSE, NULL);
-				eel_g_list_free_deep (emblem_icons);
-		
-				g_value_set_object (value, icon);
-
-				if (icon != NULL) { 
-					g_object_unref (icon);
+				emblem_pixbufs = nautilus_file_get_emblem_pixbufs (file,
+										   emblem_size,
+										   TRUE,
+										   emblems_to_ignore);
+				if (emblem_pixbufs != NULL) {
+					icon = emblem_pixbufs->data;
+					g_value_set_object (value, icon);
 				}
+				eel_gdk_pixbuf_list_free (emblem_pixbufs);
 			}
 		}
 		break;
@@ -370,25 +367,24 @@ fm_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int column
  	default:
  		if (column >= FM_LIST_MODEL_NUM_COLUMNS || column < FM_LIST_MODEL_NUM_COLUMNS + model->details->columns->len) {
 			NautilusColumn *nautilus_column;
-			char *attribute;
+			GQuark attribute;
 			nautilus_column = model->details->columns->pdata[column - FM_LIST_MODEL_NUM_COLUMNS];
 			
 			g_value_init (value, G_TYPE_STRING);
 			g_object_get (nautilus_column, 
-				      "attribute", &attribute, 
+				      "attribute_q", &attribute, 
 				      NULL);
 			if (file != NULL) {
-				str = nautilus_file_get_string_attribute_with_default (file, 
-									       attribute);
-				g_value_set_string_take_ownership (value, str);
-			} else if (!strcmp (attribute, "name")) {
+				str = nautilus_file_get_string_attribute_with_default_q (file, 
+											 attribute);
+				g_value_take_string (value, str);
+			} else if (attribute == attribute_name_q) {
 				if (file_entry->parent->loaded) {
 					g_value_set_string (value, _("(Empty)"));
 				} else {
 					g_value_set_string (value, _("Loading..."));
 				}
 			}
-			g_free (attribute);
 		} else {
 			g_assert_not_reached ();
 		}
@@ -649,10 +645,10 @@ fm_list_model_file_entry_compare_func (gconstpointer a,
 	file_entry2 = (FileEntry *)b;
 	
 	if (file_entry1->file != NULL && file_entry2->file != NULL) {
-		result = nautilus_file_compare_for_sort_by_attribute (file_entry1->file, file_entry2->file,
-							      model->details->sort_attribute,
-							      model->details->sort_directories_first,
-							      (model->details->order == GTK_SORT_DESCENDING));
+		result = nautilus_file_compare_for_sort_by_attribute_q (file_entry1->file, file_entry2->file,
+									model->details->sort_attribute,
+									model->details->sort_directories_first,
+									(model->details->order == GTK_SORT_DESCENDING));
 	} else if (file_entry1->file == NULL) {
 		return -1;
 	} else {
@@ -669,10 +665,10 @@ fm_list_model_compare_func (FMListModel *model,
 {
 	int result;
 
-	result = nautilus_file_compare_for_sort_by_attribute (file1, file2,
-							      model->details->sort_attribute,
-							      model->details->sort_directories_first,
-							      (model->details->order == GTK_SORT_DESCENDING));
+	result = nautilus_file_compare_for_sort_by_attribute_q (file1, file2,
+								model->details->sort_attribute,
+								model->details->sort_directories_first,
+								(model->details->order == GTK_SORT_DESCENDING));
 
 	return result;
 }
@@ -785,7 +781,6 @@ fm_list_model_set_sort_column_id (GtkTreeSortable *sortable, gint sort_column_id
 
 	model = (FMListModel *)sortable;
 
-	g_free (model->details->sort_attribute);
 	model->details->sort_attribute = fm_list_model_get_attribute_from_sort_column_id (model, sort_column_id);
 
 	model->details->order = order;
@@ -1316,57 +1311,55 @@ fm_list_model_set_should_sort_directories_first (FMListModel *model, gboolean so
 
 int
 fm_list_model_get_sort_column_id_from_attribute (FMListModel *model,
-						 const char *attribute)
+						 GQuark attribute)
 {
 	guint i;
 
-	if (attribute == NULL) {
+	if (attribute == 0) {
 		return -1;
 	}
 
 	/* Hack - the preferences dialog sets modification_date for some 
 	 * rather than date_modified for some reason.  Make sure that 
 	 * works. */
-	if (!strcmp (attribute, "modification_date")) {
-		attribute = "date_modified";
+	if (attribute == attribute_modification_date_q) {
+		attribute = attribute_date_modified_q;
 	}
 
 	for (i = 0; i < model->details->columns->len; i++) {
 		NautilusColumn *column;
-		char *column_attribute;
+		GQuark column_attribute;
 		
 		column = 
 			NAUTILUS_COLUMN (model->details->columns->pdata[i]);
 		g_object_get (G_OBJECT (column), 
-			      "attribute", &column_attribute, 
+			      "attribute_q", &column_attribute, 
 			      NULL);
-		if (!strcmp (column_attribute, attribute)) {
-			g_free (column_attribute);
+		if (column_attribute == attribute) {
 			return FM_LIST_MODEL_NUM_COLUMNS + i;
 		}
-		g_free (column_attribute);
 	}
 	
 	return -1;
 }
 
-char *
+GQuark
 fm_list_model_get_attribute_from_sort_column_id (FMListModel *model,
 						 int sort_column_id)
 {
 	NautilusColumn *column;
 	int index;
-	char *attribute;
+	GQuark attribute;
 	
 	index = sort_column_id - FM_LIST_MODEL_NUM_COLUMNS;
 
 	if (index < 0 || index >= model->details->columns->len) {
 		g_warning ("unknown sort column id: %d", sort_column_id);
-		return NULL;
+		return 0;
 	}
 
 	column = NAUTILUS_COLUMN (model->details->columns->pdata[index]);
-	g_object_get (G_OBJECT (column), "attribute", &attribute, NULL);
+	g_object_get (G_OBJECT (column), "attribute_q", &attribute, NULL);
 
 	return attribute;
 }
@@ -1560,7 +1553,6 @@ fm_list_model_finalize (GObject *object)
 
 	model = FM_LIST_MODEL (object);
 
-	g_free (model->details->sort_attribute);
 	g_free (model->details);
 	
 	EEL_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
@@ -1574,7 +1566,7 @@ fm_list_model_init (FMListModel *model)
 	model->details->top_reverse_map = g_hash_table_new (g_direct_hash, g_direct_equal);
 	model->details->directory_reverse_map = g_hash_table_new (g_direct_hash, g_direct_equal);
 	model->details->stamp = g_random_int ();
-	model->details->sort_attribute = NULL;
+	model->details->sort_attribute = 0;
 	model->details->columns = g_ptr_array_new ();
 }
 
@@ -1583,6 +1575,10 @@ fm_list_model_class_init (FMListModelClass *klass)
 {
 	GObjectClass *object_class;
 
+	attribute_name_q = g_quark_from_static_string ("name");
+	attribute_modification_date_q = g_quark_from_static_string ("modification_date");
+	attribute_date_modified_q = g_quark_from_static_string ("date_modified");
+	
 	object_class = (GObjectClass *)klass;
 	parent_class = g_type_class_peek_parent (klass);
 
