@@ -43,7 +43,6 @@
 #include <eel/eel-gnome-extensions.h>
 #include <eel/eel-stock-dialogs.h>
 #include <eel/eel-string.h>
-#include <eel/eel-vfs-extensions.h>
 #include <eel/eel-xml-extensions.h>
 #include <libxml/parser.h>
 #include <gtk/gtkmain.h>
@@ -52,12 +51,9 @@
 #include <libgnome/gnome-util.h>
 #include <libgnomeui/gnome-about.h>
 #include <libgnomeui/gnome-uidefs.h>
-#include <libgnomevfs/gnome-vfs-file-info.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
-#include <libgnomevfs/gnome-vfs-ops.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
+#include <libnautilus-private/nautilus-global-preferences.h>
 #include <libnautilus-private/nautilus-ui-utilities.h>
-#include <libnautilus-private/nautilus-icon-factory.h>
 #include <libnautilus-private/nautilus-undo-manager.h>
 #include <libnautilus-private/nautilus-search-engine.h>
 #include <libnautilus-private/nautilus-signaller.h>
@@ -65,6 +61,7 @@
 #define MENU_PATH_HISTORY_PLACEHOLDER			"/MenuBar/Other Menus/Go/History Placeholder"
 
 #define RESPONSE_FORGET		1000
+#define MENU_ITEM_MAX_WIDTH_CHARS 32
 
 static void                  schedule_refresh_go_menu                      (NautilusNavigationWindow   *window);
 
@@ -114,6 +111,7 @@ forget_history_if_confirmed (NautilusWindow *window)
 		 * implemented this feature so I could use this joke. 
 		 */
 		prompt = _("Are you sure you want to forget history?");
+		/* Translators: This is part of a joke and is paired with "Are you sure you want to forget history?" */
 		detail = _("If you do, you will be doomed to repeat it.");
 	} else {
 		prompt = _("Are you sure you want to clear the list "
@@ -233,6 +231,19 @@ nautilus_navigation_window_update_show_hide_menu_items (NautilusNavigationWindow
 				      nautilus_navigation_window_status_bar_showing (window));
 }
 
+void
+nautilus_navigation_window_update_spatial_menu_item (NautilusNavigationWindow *window) 
+{
+	GtkAction *action;
+
+	g_assert (NAUTILUS_IS_NAVIGATION_WINDOW (window));
+
+	action = gtk_action_group_get_action (window->details->navigation_action_group,
+					      NAUTILUS_ACTION_FOLDER_WINDOW);
+	gtk_action_set_visible (action,
+				!eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER));
+}
+
 static void
 action_add_bookmark_callback (GtkAction *action,
 			      gpointer user_data)
@@ -276,14 +287,14 @@ nautilus_navigation_window_remove_go_menu_items (NautilusNavigationWindow *windo
 
 static void
 show_bogus_history_window (NautilusWindow *window,
-			    NautilusBookmark *bookmark)
+			   NautilusBookmark *bookmark)
 {
-	char *uri;
+	GFile *file;
 	char *uri_for_display;
 	char *detail;
 
-	uri = nautilus_bookmark_get_uri (bookmark);
-	uri_for_display = eel_format_uri_for_display (uri);
+	file = nautilus_bookmark_get_location (bookmark);
+	uri_for_display = g_file_get_parse_name (file);
 	
 	detail = g_strdup_printf (_("The location \"%s\" does not exist."), uri_for_display);
 
@@ -291,11 +302,28 @@ show_bogus_history_window (NautilusWindow *window,
 				 detail,
 				 GTK_WINDOW (window));
 
-	g_free (uri);
+	g_object_unref (file);
 	g_free (uri_for_display);
 	g_free (detail);
 }
 
+static void
+connect_proxy_cb (GtkActionGroup *action_group,
+                  GtkAction *action,
+                  GtkWidget *proxy,
+                  gpointer dummy)
+{
+	GtkLabel *label;
+
+	if (!GTK_IS_MENU_ITEM (proxy))
+		return;
+
+	label = GTK_LABEL (GTK_BIN (proxy)->child);
+
+	gtk_label_set_use_underline (label, FALSE);
+	gtk_label_set_ellipsize (label, PANGO_ELLIPSIZE_END);
+	gtk_label_set_max_width_chars (label, MENU_ITEM_MAX_WIDTH_CHARS);
+}
 
 /**
  * refresh_go_menu:
@@ -322,6 +350,8 @@ refresh_go_menu (NautilusNavigationWindow *window)
 
 	window->details->go_menu_merge_id = gtk_ui_manager_new_merge_id (ui_manager);
 	window->details->go_menu_action_group = gtk_action_group_new ("GoMenuGroup");
+	g_signal_connect (window->details->go_menu_action_group, "connect-proxy",
+			  G_CALLBACK (connect_proxy_cb), NULL);
 
 	gtk_ui_manager_insert_action_group (ui_manager,
 					    window->details->go_menu_action_group,
@@ -377,12 +407,9 @@ static void
 nautilus_navigation_window_initialize_go_menu (NautilusNavigationWindow *window)
 {
 	/* Recreate bookmarks part of menu if history list changes
-	 * or if icon theme changes.
 	 */
 	g_signal_connect_object (nautilus_signaller_get_current (), "history_list_changed",
 				 G_CALLBACK (schedule_refresh_go_menu), window, G_CONNECT_SWAPPED);
-	g_signal_connect_object (nautilus_icon_factory_get (), "icons_changed",
-				 G_CALLBACK (schedule_refresh_go_menu), window, G_CONNECT_SWAPPED);
 }
 
 static void
@@ -400,6 +427,26 @@ action_new_window_callback (GtkAction *action,
 	nautilus_window_go_home (new_window);
 }
 
+static void
+action_folder_window_callback (GtkAction *action,
+			       gpointer user_data)
+{
+	NautilusWindow *current_window;
+	GFile *current_location;
+
+	current_window = NAUTILUS_WINDOW (user_data);
+	current_location = nautilus_window_get_location (current_window);
+	nautilus_application_present_spatial_window (
+			current_window->application,
+			current_window,
+			NULL,
+			current_location,
+			gtk_window_get_screen (GTK_WINDOW (current_window)));
+	if (current_location != NULL) {
+		g_object_unref (current_location);
+	}
+}
+
 static void
 action_go_to_location_callback (GtkAction *action,
 				gpointer user_data)
@@ -425,9 +472,12 @@ action_search_callback (GtkAction *action,
 static const GtkActionEntry navigation_entries[] = {
   { "Go", NULL, N_("_Go") },               /* name, stock id, label */
   { "Bookmarks", NULL, N_("_Bookmarks") },               /* name, stock id, label */
-  { "New Window", "window-new", N_("Open New _Window"),               /* name, stock id, label */
+  { "New Window", "window-new", N_("New _Window"),               /* name, stock id, label */
     "<control>N", N_("Open another Nautilus window for the displayed location"),
     G_CALLBACK (action_new_window_callback) },
+  { "Folder Window", "folder", N_("Open Folder W_indow"),               /* name, stock id, label */
+    NULL, N_("Open a folder window for the displayed location"),
+    G_CALLBACK (action_folder_window_callback) },
   { "Close All Windows", NULL, N_("Close _All Windows"),               /* name, stock id, label */
     "<control><shift>W", N_("Close all Navigation windows"),
     G_CALLBACK (action_close_all_windows_callback) },
@@ -457,7 +507,7 @@ static const GtkToggleActionEntry navigation_toggle_entries[] = {
     TRUE}, /* is_active */
   { "Show Hide Sidebar", NULL,                 /* name, stock id */
     N_("_Side Pane"), "F9",                    /* label, accelerator */     
-    N_("Change the visibility of this window's sidebar"),                       /* tooltip */
+    N_("Change the visibility of this window's side pane"),                       /* tooltip */
     G_CALLBACK (action_show_hide_sidebar_callback),
     TRUE}, /* is_active */
   { "Show Hide Location Bar", NULL,                 /* name, stock id */
@@ -552,6 +602,7 @@ nautilus_navigation_window_initialize_menus (NautilusNavigationWindow *window)
 	gtk_ui_manager_add_ui_from_string (ui_manager, ui, -1, NULL);
 
 	nautilus_navigation_window_update_show_hide_menu_items (window);
+	nautilus_navigation_window_update_spatial_menu_item (window);
 
         nautilus_navigation_window_initialize_go_menu (window);
 }
